@@ -2,9 +2,11 @@
 Addapted from Python standard module test_support.
 """
 
+import contextlib
 import unittest
 import pdb
 import sys
+import tracemalloc
 
 
 class Error(Exception):
@@ -66,7 +68,7 @@ def run_suite(suite, testclass=None):
 
 def run_unittest(testclass, debug=0):
     """Run tests from a unittest.TestCase-derived class."""
-    suite = unittest.makeSuite(testclass)
+    suite = unittest.defaultTestLoader.loadTestsFromTestCase(testclass)
     if debug:
         suite.debug()
     else:
@@ -91,6 +93,7 @@ class TestCase(unittest.TestCase):
         self.View = self.heapy.View
         self.iso = self.Use.iso
         self.idset = self.Use.idset
+        self.version_info = sys.version_info
 
     def aseq(self, a, b, cont=0):
         if a != b:
@@ -114,3 +117,24 @@ class TestCase(unittest.TestCase):
 
     def tearDown(self):
         pass
+
+    @contextlib.contextmanager
+    def tracemalloc_state(enabled=True):
+        orig_enabled = tracemalloc.is_tracing()
+
+        def set_enabled(new_enabled):
+            cur_enabled = tracemalloc.is_tracing()
+            if cur_enabled == new_enabled:
+                return
+
+            if new_enabled:
+                tracemalloc.start()
+            else:
+                tracemalloc.stop()
+
+        set_enabled(enabled)
+
+        try:
+            yield
+        finally:
+            set_enabled(orig_enabled)
