@@ -111,8 +111,8 @@ ng_dealloc(PyObject *v)
 {
     NyNodeGraphObject *ng = (void *)v;
     Py_ssize_t i;
-    Py_TRASHCAN_SAFE_BEGIN(v)
     PyObject_GC_UnTrack(v);
+    Py_TRASHCAN_BEGIN(v, ng_dealloc)
     ng_gc_clear(ng);
     for (i = 0; i < ng->used_size; i++) {
         Py_DECREF(ng->edges[i].src);
@@ -120,8 +120,7 @@ ng_dealloc(PyObject *v)
     }
     PyMem_FREE(ng->edges);
     Py_TYPE(ng)->tp_free(v);
-    Py_TRASHCAN_SAFE_END(v)
-
+    Py_TRASHCAN_END
 }
 
 static int
@@ -149,8 +148,23 @@ NyNodeGraph_AddEdge(NyNodeGraphObject *ng, PyObject *src, PyObject *tgt)
         ng->edges[ng->used_size-1].tgt == tgt)
         return 0;
 
-    assert(Py_REFCNT(src) < 0xa000000 && (Py_uintptr_t)Py_TYPE(src) > 0x1000);
-    assert(Py_REFCNT(tgt) < 0xa000000 && (Py_uintptr_t)Py_TYPE(tgt) > 0x1000);
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    assert((Py_uintptr_t)Py_TYPE(src) > 0x1000 &&
+            (Py_REFCNT(src) < 0xa000000 || _Py_IsImmortal(src)));
+    assert((Py_uintptr_t)Py_TYPE(tgt) > 0x1000 &&
+            (Py_REFCNT(tgt) < 0xa000000 || _Py_IsImmortal(tgt)));
+#elif PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+    /* Py >= 3.11 _PyObject_IMMORTAL_INIT sets initial refcount of 999999999 */
+    assert((Py_uintptr_t)Py_TYPE(src) > 0x1000 &&
+            (Py_REFCNT(src) < 0xa000000 ||
+             (Py_REFCNT(src) >= 999999999 && Py_REFCNT(src) < 999999999 + 0xa000000)));
+    assert((Py_uintptr_t)Py_TYPE(tgt) > 0x1000 &&
+            (Py_REFCNT(tgt) < 0xa000000 ||
+             (Py_REFCNT(tgt) >= 999999999 && Py_REFCNT(tgt) < 999999999 + 0xa000000)));
+#else
+    assert((Py_uintptr_t)Py_TYPE(src) > 0x1000 && Py_REFCNT(src) < 0xa000000);
+    assert((Py_uintptr_t)Py_TYPE(tgt) > 0x1000 && Py_REFCNT(tgt) < 0xa000000);
+#endif
 
     if (ng->used_size >= ng->allo_size) {
         Py_ssize_t allo = roundupsize(ng->used_size + 1);
