@@ -103,17 +103,11 @@ char rootstate_doc[] =
 "that is the number of frames it has before it in call order.\n"
 ;
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION == 8
-# define Py_BUILD_CORE
+
+#define Py_BUILD_CORE
 /* PyInterpreterState */
-#  include <internal/pycore_pystate.h>
-# undef Py_BUILD_CORE
-#elif PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
-# define Py_BUILD_CORE
-/* PyInterpreterState */
-#  include <internal/pycore_interp.h>
-# undef Py_BUILD_CORE
-#endif
+# include <internal/pycore_interp.h>
+#undef Py_BUILD_CORE
 
 #define THREAD_ID(ts)    (ts->thread_id)
 
@@ -153,9 +147,15 @@ static struct PyMemberDef is_members[] = {
     MEMBER(sysdict),
     MEMBER(builtins),
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    RENAMEMEMBER(codecs.search_path, codec_search_path),
+    RENAMEMEMBER(codecs.search_cache, codec_search_cache),
+    RENAMEMEMBER(codecs.error_registry, codec_error_registry),
+#else
     MEMBER(codec_search_path),
     MEMBER(codec_search_cache),
     MEMBER(codec_error_registry),
+#endif
 
     MEMBER(dict),
     MEMBER(builtins_copy),
@@ -170,6 +170,12 @@ static struct PyMemberDef is_members[] = {
 #endif
 
     MEMBER(audit_hooks),
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    MEMBER(optimizer),
+    MEMBER(executor_list_head), // TODO: Iterate this list
+#endif
+
     {0} /* Sentinel */
 };
 
@@ -211,37 +217,47 @@ static struct PyMemberDef ts_members[] = {
     MEMBER(async_gen_finalizer),
 
     MEMBER(context),
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    MEMBER(previous_executor),
+    MEMBER(threading_local_key),
+    // threading_local_sentinel not included
+#endif
+
     {0} /* Sentinel */
 };
 
 #undef MEMBER
 #undef RENAMEMEMBER
 
-#define ISATTR(name)                                                                  \
+#define ISATTR(name) do {                                                             \
     if ((PyObject *)is->name == r->tgt) {                                             \
         if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_%s", isno, #name), r)) \
             return 1;                                                                 \
-    }
+    }                                                                                 \
+} while (0)
 
-#define RENAMEISATTR(name, newname)                                                                  \
-    if ((PyObject *)is->name == r->tgt) {                                             \
+#define RENAMEISATTR(name, newname) do {                                                 \
+    if ((PyObject *)is->name == r->tgt) {                                                \
         if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_%s", isno, #newname), r)) \
-            return 1;                                                                 \
-    }
+            return 1;                                                                    \
+    }                                                                                    \
+} while (0)
 
-#define TSATTR(name)                                                                                      \
+#define TSATTR(name) do {                                                                                 \
     if ((PyObject *)ts->name == r->tgt) {                                                                 \
         if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_t%lu_%s", isno, THREAD_ID(ts), #name), r)) \
             return 1;                                                                                     \
-    }
-
-#define RENAMETSATTR(name, newname)                                                                          \
+    }                                                                                                     \
+} while (0)
+#define RENAMETSATTR(name, newname) do {                                                                     \
     if ((PyObject *)ts->name == r->tgt) {                                                                    \
         if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_t%lu_%s", isno, THREAD_ID(ts), #newname), r)) \
             return 1;                                                                                        \
-    }
+    }                                                                                                        \
+} while (0)
 
-#define ISATTR_DIR(name)                                \
+#define ISATTR_DIR(name) do {                           \
     attr = PyUnicode_FromFormat("i%d_%s", isno, #name); \
     if (attr) {                                         \
         if (PyList_Append(list, attr)) {                \
@@ -250,8 +266,9 @@ static struct PyMemberDef ts_members[] = {
         }                                               \
         Py_DECREF(attr);                                \
     }                                                   \
+} while (0)
 
-#define TSATTR_DIR(name)                                                    \
+#define TSATTR_DIR(name) do {                                               \
     attr = PyUnicode_FromFormat("i%d_t%lu_%s", isno, THREAD_ID(ts), #name); \
     if (attr) {                                                             \
         if (PyList_Append(list, attr)) {                                    \
@@ -260,7 +277,7 @@ static struct PyMemberDef ts_members[] = {
         }                                                                   \
         Py_DECREF(attr);                                                    \
     }                                                                       \
-
+} while (0)
 
 static int
 rootstate_relate(NyHeapRelate *r)
@@ -276,10 +293,8 @@ rootstate_relate(NyHeapRelate *r)
     for (is = PyInterpreterState_Head(), isno--;
          is;
          is = PyInterpreterState_Next(is), isno--) {
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
         if (is != PyInterpreterState_Get())
             continue;
-#endif
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
         RENAMEISATTR(imports.modules, modules);
         RENAMEISATTR(imports.modules_by_index, modules_by_index);
@@ -295,9 +310,15 @@ rootstate_relate(NyHeapRelate *r)
         ISATTR(sysdict);
         ISATTR(builtins);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+        RENAMEISATTR(codecs.search_path, codec_search_path);
+        RENAMEISATTR(codecs.search_cache, codec_search_cache);
+        RENAMEISATTR(codecs.error_registry, codec_error_registry);
+#else
         ISATTR(codec_search_path);
         ISATTR(codec_search_cache);
         ISATTR(codec_error_registry);
+#endif
 
         ISATTR(dict);
         ISATTR(builtins_copy);
@@ -313,6 +334,11 @@ rootstate_relate(NyHeapRelate *r)
 
         ISATTR(audit_hooks);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+        ISATTR(optimizer);
+        ISATTR(executor_list_head);
+#endif
+
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
         ts = is->threads.head;
 #else
@@ -373,6 +399,11 @@ rootstate_relate(NyHeapRelate *r)
             TSATTR(async_gen_finalizer);
 
             TSATTR(context);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+            TSATTR(previous_executor);
+            TSATTR(threading_local_key);
+#endif
         }
     }
     return 0;
@@ -391,10 +422,8 @@ rootstate_traverse(NyHeapTraverse *ta)
     for (is = PyInterpreterState_Head(); is; is = PyInterpreterState_Next(is)) {
         if (hv->is_hiding_calling_interpreter && is == bts->interp)
             continue;
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
         if (is != PyInterpreterState_Get())
             continue;
-#endif
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
         Py_VISIT(is->imports.modules);
         // Not traversing through this because it is of the same level as
@@ -412,9 +441,15 @@ rootstate_traverse(NyHeapTraverse *ta)
         Py_VISIT(is->sysdict);
         Py_VISIT(is->builtins);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+        Py_VISIT(is->codecs.search_path);
+        Py_VISIT(is->codecs.search_cache);
+        Py_VISIT(is->codecs.error_registry);
+#else
         Py_VISIT(is->codec_search_path);
         Py_VISIT(is->codec_search_cache);
         Py_VISIT(is->codec_error_registry);
+#endif
 
         Py_VISIT(is->dict);
         Py_VISIT(is->builtins_copy);
@@ -430,6 +465,11 @@ rootstate_traverse(NyHeapTraverse *ta)
 
         Py_VISIT(is->audit_hooks);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+        Py_VISIT(is->optimizer);
+        Py_VISIT(is->executor_list_head);
+#endif
+
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
         ts = is->threads.head;
 #else
@@ -473,6 +513,11 @@ rootstate_traverse(NyHeapTraverse *ta)
             Py_VISIT(ts->async_gen_finalizer);
 
             Py_VISIT(ts->context);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+            Py_VISIT(ts->previous_executor);
+            Py_VISIT(ts->threading_local_key);
+#endif
         }
     }
     return 0;
@@ -516,10 +561,8 @@ rootstate_getattr(PyObject *obj, PyObject *name)
         for (is = PyInterpreterState_Head(), countis = 0;
              is;
              is = PyInterpreterState_Next(is), countis++) {
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
             if (is != PyInterpreterState_Get())
                 continue;
-#endif
             int isno = numis - countis - 1;
             if (isno == ino) {
                 if (sscanf(s, "t%lu_%n", &tno, &n) == 1) {
@@ -613,10 +656,8 @@ rootstate_getattr(PyObject *obj, PyObject *name)
         for (is = PyInterpreterState_Head(), countis = 0;
              is;
              is = PyInterpreterState_Next(is), countis++) {
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
             if (is != PyInterpreterState_Get())
                 continue;
-#endif
             int isno = numis - countis - 1;
 
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
@@ -679,10 +720,8 @@ rootstate_dir(PyObject *self, PyObject *args)
     for (is = PyInterpreterState_Head(), isno--;
          is;
          is = PyInterpreterState_Next(is), isno--) {
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
         if (is != PyInterpreterState_Get())
             continue;
-#endif
         ISATTR_DIR(modules);
         ISATTR_DIR(modules_by_index);
         ISATTR_DIR(importlib);
@@ -709,6 +748,11 @@ rootstate_dir(PyObject *self, PyObject *args)
 
         ISATTR_DIR(audit_hooks);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+        ISATTR_DIR(optimizer);
+        ISATTR_DIR(executor_list_head);
+#endif
+
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
         ts = is->threads.head;
 #else
@@ -767,6 +811,11 @@ rootstate_dir(PyObject *self, PyObject *args)
             TSATTR_DIR(async_gen_finalizer);
 
             TSATTR_DIR(context);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+            TSATTR_DIR(previous_executor);
+            TSATTR_DIR(threading_local_key);
+#endif
         }
     }
 
@@ -806,5 +855,6 @@ PyTypeObject NyRootState_Type = {
 
 PyObject _Ny_RootStateStruct = {
     _PyObject_EXTRA_INIT
-    1, &NyRootState_Type
+    .ob_refcnt = 1,
+    .ob_type = &NyRootState_Type,
 };
