@@ -29,13 +29,15 @@
 #include "stdtypes.h"
 
 
-#define GATTR(obj, name, rel) if ((PyObject *)(obj) == r->tgt &&        \
-    (r->visit(rel, PyUnicode_FromString(#name), r))) \
-        return 1;
+#define GATTR(obj, name, rel) do {                           \
+    if ((PyObject *)(obj) == r->tgt &&                       \
+            (r->visit(rel, PyUnicode_FromString(#name), r))) \
+        return 1;                                            \
+} while (0)
 
-#define ATTR(name) GATTR(v->name, name, NYHR_ATTRIBUTE)
-#define RENAMEATTR(name, newname) GATTR(v->name, newname, NYHR_ATTRIBUTE)
-#define INTERATTR(name) GATTR(v->name, name, NYHR_INTERATTR)
+#define ATTR(name) GATTR(v->name, name, NYHR_ATTRIBUTE);
+#define RENAMEATTR(name, newname) GATTR(v->name, newname, NYHR_ATTRIBUTE);
+#define INTERATTR(name) GATTR(v->name, name, NYHR_INTERATTR);
 
 extern PyObject *_hiding_tag__name;
 
@@ -74,6 +76,18 @@ dict_traverse(NyHeapTraverse *ta)
     PyObject *v = (void *)ta->obj;
     if (PyDict_GetItem(v, _hiding_tag__name) == ta->_hiding_tag_)
         return 0;
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    // Python 3.13 no longer traverses object .__dict__ values.
+    visitproc visit = ta->visit;
+    void *arg = ta->arg;
+    Py_ssize_t i = 0;
+    PyObject *pv;
+
+    while (PyDict_Next(v, &i, NULL, &pv))
+        Py_VISIT(pv);
+#endif
+
     return Py_TYPE(v)->tp_traverse(ta->obj, ta->visit, ta->arg);
 }
 
@@ -160,17 +174,17 @@ static int
 function_relate(NyHeapRelate *r)
 {
     PyFunctionObject *v = (void *)r->src;
-    RENAMEATTR(func_code, __code__)
-    RENAMEATTR(func_globals, __globals__)
-    RENAMEATTR(func_module, __module__)
-    RENAMEATTR(func_defaults, __defaults__)
-    RENAMEATTR(func_kwdefaults, __kwdefaults__)
-    RENAMEATTR(func_doc, __doc__)
-    RENAMEATTR(func_name, __name__)
-    RENAMEATTR(func_dict, __dict__)
-    RENAMEATTR(func_closure, __closure__)
-    RENAMEATTR(func_annotations, __annotations__)
-    RENAMEATTR(func_qualname, __qualname__)
+    RENAMEATTR(func_code, __code__);
+    RENAMEATTR(func_globals, __globals__);
+    RENAMEATTR(func_module, __module__);
+    RENAMEATTR(func_defaults, __defaults__);
+    RENAMEATTR(func_kwdefaults, __kwdefaults__);
+    RENAMEATTR(func_doc, __doc__);
+    RENAMEATTR(func_name, __name__);
+    RENAMEATTR(func_dict, __dict__);
+    RENAMEATTR(func_closure, __closure__);
+    RENAMEATTR(func_annotations, __annotations__);
+    RENAMEATTR(func_qualname, __qualname__);
     return dict_relate_kv(r, v->func_dict, NYHR_HASATTR, NYHR_ATTRIBUTE);
 }
 
@@ -218,7 +232,12 @@ frame_relate(NyHeapRelate *r)
 #else
     PyFrameObject *iv = v;
 #endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    PyCodeObject *co = iv->f_executable && PyCode_Check(iv->f_executable) ?
+                       (PyCodeObject *)iv->f_executable : NULL;
+#else
     PyCodeObject *co = iv->f_code;
+#endif
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 11
     Py_ssize_t ncells = PyTuple_GET_SIZE(co->co_cellvars);
     Py_ssize_t nlocals = co->co_nlocals;;
@@ -236,16 +255,37 @@ frame_relate(NyHeapRelate *r)
 #endif
     ATTR(f_back)
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
-    GATTR(iv->f_funcobj, f_funcobj, NYHR_INTERATTR)
+    GATTR(iv->f_funcobj, f_funcobj, NYHR_INTERATTR);
 #elif PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
-    GATTR(iv->f_func, f_func, NYHR_INTERATTR)
+    GATTR(iv->f_func, f_func, NYHR_INTERATTR);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    if (iv->f_executable && PyCode_Check(iv->f_executable))
+        GATTR(iv->f_executable, f_code, NYHR_ATTRIBUTE);
+    else
+        GATTR(iv->f_executable, f_executable, NYHR_INTERATTR);
+#else
+    GATTR(iv->f_code, f_code, NYHR_ATTRIBUTE);
+#endif
+    GATTR(iv->f_builtins, f_builtins, NYHR_ATTRIBUTE);
+    GATTR(iv->f_globals, f_globals, NYHR_ATTRIBUTE);
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    GATTR(iv->f_locals, f_locals, NYHR_INTERATTR);
+#else
+    GATTR(iv->f_locals, f_locals, NYHR_ATTRIBUTE);
 #endif
-    GATTR(iv->f_code, f_code, NYHR_ATTRIBUTE)
-    GATTR(iv->f_builtins, f_builtins, NYHR_ATTRIBUTE)
-    GATTR(iv->f_globals, f_globals, NYHR_ATTRIBUTE)
-    GATTR(iv->f_locals, f_locals, NYHR_ATTRIBUTE)
     ATTR(f_trace)
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    ATTR(f_extra_locals)
+    ATTR(f_locals_cache)
+#endif
+
+    // FIXME: Not sure if there's anything one can do about optimized frames,
+    // need testing.
+    if (!co)
+        return 0;
+
     /* locals */
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
     Py_ssize_t i;
@@ -322,10 +362,16 @@ frame_traverse(NyHeapTraverse *ta) {
 #else
     PyFrameObject *iv = v;
 #endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    PyCodeObject *co = iv->f_executable && PyCode_Check(iv->f_executable) ?
+                       (PyCodeObject *)iv->f_executable : NULL;
+#else
     PyCodeObject *co = iv->f_code;
+#endif
     int i;
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
-    for (i = 0; i < co->co_nlocalsplus; i++) {
+    // FIXME: What happens if JIT?
+    for (i = 0; co && i < co->co_nlocalsplus; i++) {
         _PyLocals_Kind kind = _PyLocals_GetKind(co->co_localspluskinds, i);
         PyObject *name = PyTuple_GET_ITEM(co->co_localsplusnames, i);
         if (kind & CO_FAST_LOCAL && strcmp(PyUnicode_AsUTF8(name), "_hiding_tag_") == 0) {
@@ -363,14 +409,28 @@ frame_traverse(NyHeapTraverse *ta) {
 #else
     Py_VISIT(iv->f_func);
 #endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    Py_VISIT(iv->f_executable);
+#else
     Py_VISIT(iv->f_code);
+#endif
     Py_VISIT(iv->f_builtins);
     Py_VISIT(iv->f_globals);
     Py_VISIT(iv->f_locals);
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    Py_VISIT(v->f_extra_locals);
+    Py_VISIT(v->f_locals_cache);
+#endif
 
     /* locals */
-    for (i = 0; i < co->co_nlocalsplus; i++)
-        Py_VISIT(iv->localsplus[i]);
+    if (!co) {
+        // FIXME: is it okay to assume stacktop is always valid when !co?
+        for (i = 0; i < iv->stacktop; i++)
+            Py_VISIT(iv->localsplus[i]);
+    } else {
+        for (i = 0; i < co->co_nlocalsplus; i++)
+            Py_VISIT(iv->localsplus[i]);
+    }
 
     return 0;
 #else
@@ -494,6 +554,52 @@ code_relate(NyHeapRelate *r)
     return 0;
 }
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+# if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+typedef managed_static_type_state ny_static_type_state;
+# else
+typedef static_builtin_state ny_static_type_state;
+# endif
+
+static ny_static_type_state *NyStaticType_GetState(PyTypeObject *self)
+{
+    // FIXME: interpreter probably should be a argument,
+    // but with per-interp GIL, it's only safe to traverse
+    // current interpreter anyways.
+    PyInterpreterState *is = PyInterpreterState_Get();
+
+    assert(self->tp_flags & _Py_TPFLAGS_STATIC_BUILTIN);
+
+# if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 13
+    managed_static_type_state *state;
+    size_t index;
+
+    index = (size_t)self->tp_subclasses - 1;
+
+    // FIXME: These constants may be subject to change within a Python version
+    if (index <= _Py_MAX_MANAGED_STATIC_BUILTIN_TYPES) {
+        state = &is->types.builtins.initialized[index];
+        if (state->type == self)
+            return state;
+    }
+    if (index <= _Py_MAX_MANAGED_STATIC_EXT_TYPES) {
+        state = &is->types.for_extensions.initialized[index];
+        if (state->type == self)
+            return state;
+    }
+
+    PyErr_Format(PyExc_RuntimeError,
+        "Unable to find managed_static_type_state for %R", self);
+    return NULL;
+# else
+    size_t index;
+
+    index = (size_t)self->tp_subclasses - 1;
+    return &is->types.builtins[index];
+# endif
+}
+#endif
+
 /* type_traverse adapted from typeobject.c from 2.4.2
    except:
    * I removed the check for heap type
@@ -509,15 +615,9 @@ type_traverse(NyHeapTraverse *ta)
 
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
     if (type->tp_flags & _Py_TPFLAGS_STATIC_BUILTIN) {
-        // TODO: interpreter probably should be a argument,
-        // but with per-interp GIL, it's only safe to traverse
-        // current interpreter anyways.
-        PyInterpreterState *is = PyInterpreterState_Get();
-        static_builtin_state *state;
-        size_t index;
-
-        index = (size_t)type->tp_subclasses - 1;
-        state = &is->types.builtins[index];
+        ny_static_type_state *state = NyStaticType_GetState(type);
+        if (!state)
+            return -1;
 
         Py_VISIT(state->tp_dict);
         Py_VISIT(state->tp_subclasses);
@@ -539,15 +639,11 @@ type_traverse(NyHeapTraverse *ta)
     Py_VISIT(((PyHeapTypeObject *)type)->ht_slots);
     Py_VISIT(((PyHeapTypeObject *)type)->ht_qualname);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
     Py_VISIT(((PyHeapTypeObject *)type)->ht_module);
-#endif
     return 0;
 }
 
 
-
-
 static int
 type_relate(NyHeapRelate *r)
 {
@@ -556,15 +652,9 @@ type_relate(NyHeapRelate *r)
 
 #if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
     if (type->tp_flags & _Py_TPFLAGS_STATIC_BUILTIN) {
-        // TODO: interpreter probably should be a argument,
-        // but with per-interp GIL, it's only safe to traverse
-        // current interpreter anyways.
-        PyInterpreterState *is = PyInterpreterState_Get();
-        static_builtin_state *state;
-        size_t index;
-
-        index = (size_t)type->tp_subclasses - 1;
-        state = &is->types.builtins[index];
+        ny_static_type_state *state = NyStaticType_GetState(type);
+        if (!state)
+            return -1;
 
 #define v state
         RENAMEATTR(tp_dict, __dict__);
@@ -591,9 +681,7 @@ type_relate(NyHeapRelate *r)
     RENAMEATTR(ht_slots, __slots__);
     RENAMEATTR(ht_qualname, __qualname__);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
     INTERATTR(ht_module);
-#endif
     return 0;
 #undef v
 }
