@@ -73,7 +73,7 @@ static NyHeapDef nysets_heapdefs[] = {
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     MODNAME,
-    NULL,
+    PyDoc_STR(sets_doc),
     -1,
     module_methods
 };
@@ -96,8 +96,6 @@ INITFUNC (void)
         goto Error;
     if (fsb_dx_nynodeset_init(m) == -1)
         goto Error;
-    if (PyDict_SetItemString(d, "__doc__", PyUnicode_FromString(sets_doc)) == -1)
-        goto Error;
     if (PyDict_SetItemString(d,
              "_NyHeapDefs_",
              PyCapsule_New(
